@@ -74,6 +74,11 @@ class TML:
     def guid(self, new_guid: str):
         self.tml['guid'] = new_guid
 
+    @property
+    def tables(self):
+        key = "tables"
+        return self._first_level_property(key)
+
     def remove_calendars(self):
         """Removes any references to customer calendars, which aren't supported on Embrace tables."""
         # The above comment may no longer be true in recent versions
@@ -94,6 +99,25 @@ class TML:
                 strval += f"{k}: {v}\n"
         return strval
 
+    # TML files do not include the GUIDs of related objects. But this can be retrieved from export_associated=true
+    # option of the tml/export command. This takes a dictionary of Object Name: GUID generated from that REST API
+    def add_fqns_from_name_guid_map(self, name_guid_map: Dict):
+        if self.tables is not None:
+            for a in self.tables:
+                table_name = a['name']
+                if table_name in name_guid_map:
+                    a['fqn'] = name_guid_map[table_name]
+
+    def replace_fqns_from_map(self, parent_child_guid_map: Dict, ignore_missing=False):
+        if self.tables is not None:
+            for a in self.tables:
+                if 'fqn' in a:
+                    parent_fqn = a['fqn']
+                    if parent_fqn in parent_child_guid_map:
+                        a['fqn'] = parent_child_guid_map[parent_fqn]
+                    else:
+                        if ignore_missing is False:
+                            raise IndexError("Object GUID {} not found in parent:child GUID map".format(parent_fqn))
 
 class Worksheet(TML):
     def __init__(self, tml_ordereddict: [typing.OrderedDict, Dict]):
@@ -121,14 +145,6 @@ worksheet:
 """
         return template.format(name=worksheet_name, table_name=table_name, table_path_id=table_path_id)
 
-    # TML files do not include the GUIDs of related objects. But this can be retrieved from export_associated=true
-    # option of the tml/export command. This takes a dictionary of Object Name: GUID generated from that REST API
-    def add_fqns_from_name_guid_map(self, name_guid_map: Dict):
-        for a in self.tables:
-            table_name = a['name']
-            if table_name in name_guid_map:
-                a['fqn'] = name_guid_map[table_name]
-
     @property
     def description(self):
         key = "description"
@@ -168,11 +184,6 @@ worksheet:
         second_level_key = "join_progressive"
         self.content[first_level_key][second_level_key] = str(new_value).lower()
 
-    @property
-    def tables(self):
-        key = "tables"
-        return self._first_level_property(key)
-
     @property
     def joins(self):
         key = "joins"
@@ -271,6 +282,14 @@ table:
         return template.format(connection_name=connection_name, name=table_name, db=db_name, schema=schema,
                                db_table=db_table)
 
+    def add_fqns_from_name_guid_map(self, name_guid_map: Dict):
+        pass
+
+    @property
+    def tables(self):
+        key = "tables"
+        return None
+
     @property
     def db_name(self):
         key = "db"
@@ -312,8 +331,11 @@ table:
     def connection_name(self):
         first_level_key = "connection"
         second_level_key = "name"
-        if second_level_key in self.content[first_level_key]:
-            return self.content[first_level_key][second_level_key]
+        if first_level_key in self.content:
+            if second_level_key in self.content[first_level_key]:
+                return self.content[first_level_key][second_level_key]
+            else:
+                return None
         else:
             return None
 
@@ -458,14 +480,6 @@ class Answer(TML):
             CANDLESTICK = 'CANDLESTICK'
         self.CHART_TYPES = ChartTypes
 
-    # TML files do not include the GUIDs of related objects. But this can be retrieved from export_associated=true
-    # option of the tml/export command. This takes a dictionary of Object Name: GUID generated from that REST API
-    def add_fqns_from_name_guid_map(self, name_guid_map: Dict):
-        for a in self.tables:
-            table_name = a['name']
-            if table_name in name_guid_map:
-                a['fqn'] = name_guid_map[table_name]
-
     @property
     def description(self):
         key = "description"
@@ -573,6 +587,11 @@ class Pinboard(TML):
         MEDIUM_SMALL = 'MEDIUM_SMALL'
         EXTRA_LARGE = 'EXTRA_LARGE'
 
+    @property
+    def tables(self):
+        key = "tables"
+        return None
+
     @property
     def visualizations(self):
         # Should these be "Answer" objects
